@@ -18,7 +18,7 @@ import SimpleCmd ((+-+))
 import SimpleCmdArgs
 import SimplePrompt (promptNonEmpty, promptPassword)
 import System.Directory
-import System.Environment.XDG.BaseDir
+import System.Environment.XDG.BaseDir (getUserConfigFile)
 import System.Exit (ExitCode (..))
 import System.FilePath
 import System.IO (BufferMode(NoBuffering), hSetBuffering, stdout)
@@ -53,11 +53,9 @@ main = do
       <$> nobuildOpt
       <*> switchWith 'U' "no-upload" "Do not upload to Hackage"
     , Subcommand "upload-haddock" "Upload candidate documentation to Hackage" $
-      upHaddockCmd False
-      <$> forceOpt "rebuild doc tarball"
+      pure (upHaddockCmd False)
     , Subcommand "publish-haddock" "Publish documentation to Hackage" $
-      upHaddockCmd True
-      <$> forceOpt "rebuild doc tarball"
+      pure (upHaddockCmd True)
     , Subcommand "build" "Do a local pristine build from the tarball" $
       pure pristineBuildCmd
     , Subcommand "version" "Show the package version from .cabal file" $
@@ -92,11 +90,13 @@ gitBool c args = do
   (== ExitSuccess) <$> P.runProcess (git c args)
 
 removeTrailingNewline :: B.ByteString -> B.ByteString
-removeTrailingNewline "" = ""
 removeTrailingNewline bs =
-  if B.last bs == '\n'
-  then B.init bs
-  else bs
+  case B.unsnoc bs of
+    Nothing -> bs
+    Just (ini,l) ->
+      if l == '\n'
+      then ini
+      else bs
 
 cmd :: String -> [String] -> IO String
 cmd c args =
@@ -285,26 +285,37 @@ getUserPassword :: IO B.ByteString
 getUserPassword = do
   cabalConfig <- do
     home <- getHomeDirectory
-    let cabalConfig = home </> ".cabal/config"
-    exists <- doesFileExist cabalConfig
-    if exists
-      then readFile cabalConfig
+    xdgConfig <- getUserConfigFile "cabal" "config"
+    have_xdg <- doesFileExist xdgConfig
+    if have_xdg
+      then readFile xdgConfig
       else do
-      putStrLn $ "Warning:" +-+ cabalConfig +-+ "not found!"
-      return ""
-  muser <- maybeGetHackage "username" False cabalConfig
-  mpasswd <- maybeGetHackage "password" True cabalConfig
+      let config = home </> ".cabal/config"
+      exists <- doesFileExist config
+      if exists
+        then readFile config
+        else do
+        putStrLn $ "Warning: no cabal" +-+ show "config" +-+ "file found!"
+        return ""
+  muser <- maybeGetUsername cabalConfig
+  mpasswd <- maybeGetPassword cabalConfig
   return $ B.pack $ unlines $ catMaybes [muser, mpasswd]
   where
-    maybeGetHackage :: String -> Bool -> String -> IO (Maybe String)
-    maybeGetHackage field hide cabalConfig =
-      if haveCabalConfigField
+    maybeGetUsername :: String -> IO (Maybe String)
+    maybeGetUsername config =
+      if haveField config "username"
         then return Nothing
-        else Just <$> prompt hide ("hackage.haskell.org" +-+ field)
-      where
-        haveCabalConfigField :: Bool
-        haveCabalConfigField =
-          any ((field ++ ":") `isPrefixOf`) (lines cabalConfig)
+        else Just <$> prompt False "hackage.haskell.org username"
+
+    maybeGetPassword :: String -> IO (Maybe String)
+    maybeGetPassword config =
+      if any (haveField config) ["password", "token", "password-command"]
+        then return Nothing
+        else Just <$> prompt True "hackage.haskell.org password"
+
+    haveField :: String -> String -> Bool
+    haveField config field =
+      any ((field ++ ":") `isPrefixOf`) (lines config)
 
 prompt :: Bool -> String -> IO String
 prompt hide s = do
@@ -312,17 +323,14 @@ prompt hide s = do
   when hide $ putChar '\n'
   return inp
 
-upHaddockCmd :: Bool -> Bool -> IO ()
-upHaddockCmd publish force = do
+upHaddockCmd :: Bool -> IO ()
+upHaddockCmd publish = do
   needProgram "cabal"
   userpassBS <- getUserPassword
   pkgid <- getPackageId
-  when force $ do
-    let tarball = "dist" </> showPkgId pkgid ++ '-' : "docs" <.> tarGzExt
-    exists <- doesFileExist tarball
-    when exists $
-      removeFile tarball
-  _out <- P.readProcessInterleaved_ (P.setStdin (P.byteStringInput userpassBS) $ P.proc "cabal" ("upload" : "--documentation" : ["--publish" | publish]))
+  let tarball = "dist-newstyle" </> showPkgId pkgid ++ '-' : "docs" <.> tarGzExt
+  cabal_ "haddock" ["--haddock-for-hackage"]
+  _out <- P.readProcessInterleaved_ (P.setStdin (P.byteStringInput userpassBS) $ P.proc "cabal" ("upload" : "--documentation" : ["--publish" | publish] ++ [tarball]))
   putStrLn $ (if publish then "Published at" else "Uploaded to") +-+ "https://hackage.haskell.org/package/" ++ showPkgId pkgid ++ if publish then "" else "/candidate"
 
 cabal_ :: String -> [String] -> IO ()
@@ -395,7 +403,7 @@ newCmd mlicense mproject = do
   -- FIXME add stack.yaml template too
   when (not haveStackCfg && isJust mstack) $ do
     -- FIXME determine last lts automatically
-    cmd_ "stack" ["init", "--verbosity", "warn", "--resolver", "lts-21"]
+    cmd_ "stack" ["init", "--verbosity", "warn", "--resolver", "lts-23"]
     sed ["/^#/d", "/^$/d"] "stack.yaml"
   haveGit <- doesDirectoryExist ".git"
   unless haveGit $ do
